# -*- coding: utf-8 -*-
# Copyright (C) 2019 fjord-technologies
# SPDX-License-Identifier: GPL-3.0-or-later
"""xtrargon_ng.modules.server"""

import logging
import os
import netifaces

from dwho.classes.modules import MODULES
from httpdis.ext.httpdis_json import HttpReqErrJson
from sonicprobe import helpers
from sonicprobe.libs.moresynchro import RWLock

from ..classes.abstract.modules import XtrargonDBModuleBase


DRBD_STATE_FILE = "/proc/drbd"

LOG             = logging.getLogger('xtrargon_ng.modules.server')


class ServerModule(XtrargonDBModuleBase):
    MODULE_NAME = 'server'

    LOCK = RWLock()

    # pylint: disable-msg=attribute-defined-outside-init
    def safe_init(self, options):
        self.lock_timeout = self.config['general']['lock_timeout']

        if not isinstance(self.config.get('db'), dict):
            self.config['db'] = {}

        if 'alive' not in self.config['db']:
            self.config['db']['alive'] = True

        if self.config['db']['alive']:
            self._init_db()

    def is_primary(self, request): # pylint: disable-msg=unused-argument
        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for reading after %s seconds" % self.lock_timeout)

        try:
            if not helpers.has_len(self.config['general'].get('primary_ifname')):
                raise HttpReqErrJson(500, "missing primary_ifname option in section general")
            return self.config['general']['primary_ifname'] in netifaces.interfaces() # pylint: disable-msg=c-extension-no-member
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()

    def has_drbd(self, request): # pylint: disable-msg=unused-argument
        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for reading after %s seconds" % self.lock_timeout)

        try:
            return os.path.isfile(DRBD_STATE_FILE)
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()

    def do_lock_db(self, request): # pylint: disable-msg=unused-argument
        if not self.LOCK.acquire_write(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for writing after %s seconds" % self.lock_timeout)

        try:
            if not self.dbconf or not self.dburi:
                raise HttpReqErrJson(400, "database is configured as not alive")

            if self.lock_db and self.lock_db.is_set():
                raise HttpReqErrJson(400, "database is already locked")

            self._init_lock_db(self.config['db'])
            self._clear_lock_db()
            self._start_lock_db()

            if self.lock_db:
                self.lock_db.set()

            return True
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()

    def get_db_status(self, request): # pylint: disable-msg=unused-argument
        if not self.LOCK.acquire_read(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for reading after %s seconds" % self.lock_timeout)

        try:
            if not self.dbconf or not self.dburi:
                raise HttpReqErrJson(400, "database is configured as not alive")

            if not self.lock_db:
                status = 'uninitialized'
            elif self.lock_db.is_set():
                status = 'locked'
            else:
                status = 'unlocked'

            return {'status': status}
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()

    def do_unlock_db(self, request): # pylint: disable-msg=unused-argument
        if not self.LOCK.acquire_write(self.lock_timeout):
            raise HttpReqErrJson(503, "unable to take LOCK for writing after %s seconds" % self.lock_timeout)

        try:
            if not self.dbconf or not self.dburi:
                raise HttpReqErrJson(400, "database is configured as not alive")

            if not self.lock_db:
                raise HttpReqErrJson(400, "database lock isn't initialized")

            if not self.lock_db.is_set():
                raise HttpReqErrJson(400, "database isn't locked")

            self._clear_lock_db()

            return True
        except HttpReqErrJson:
            raise
        except Exception as e:
            LOG.exception(e)
            raise HttpReqErrJson(503, repr(e))
        finally:
            self.LOCK.release()


if __name__ != "__main__":
    def _start():
        MODULES.register(ServerModule())
    _start()
